package cn.iocoder.yudao.module.system.service.sms;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.sms.core.client.SmsClient;
import cn.iocoder.yudao.framework.sms.core.client.SmsClientFactory;
import cn.iocoder.yudao.framework.sms.core.client.dto.SmsLogDTO;
import cn.iocoder.yudao.framework.sms.core.enums.ReceiveStatusEnum;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.smsNode.SmsNodeCreateReqVO;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.smsNode.SmsNodeQueryVO;
import cn.iocoder.yudao.module.system.controller.admin.sms.vo.smsNode.SmsNodeUpdateReqVO;
import cn.iocoder.yudao.module.system.convert.sms.SmsLogConvert;
import cn.iocoder.yudao.module.system.convert.sms.SmsNodeConvert;
import cn.iocoder.yudao.module.system.dal.dataobject.sms.SmsLogDO;
import cn.iocoder.yudao.module.system.dal.dataobject.sms.SmsNodeDO;
import cn.iocoder.yudao.module.system.dal.mysql.sms.SmsNodeMapper;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.apollo.core.constants.CacheConstants.SYSTEM_SMS_NODE_KEY;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.SMS_NODE_NOT_EXISTS;

/**
 * 短信节点 Service 实现类
 *
 * @author jayden
 */
@Slf4j
@Service
@Validated
public class SmsNodeServiceImpl extends AbstractService<SmsNodeMapper, SmsNodeDO> implements SmsNodeService {

    @Resource
    private SmsNodeMapper smsNodeMapper;

    @Autowired
    private RedisHelper redisHelper;

    @Autowired
    private SmsLogService smsLogService;

    @Autowired
    private SmsClientFactory smsClientFactory;

    @Autowired
    private SmsSendService smsSendService;

    /**
     * 项目启动时，初始化参数到缓存
     */
    @PostConstruct
    public void init() {
        loadingConfigCache();
    }

    /**
     * 加载参数缓存数据
     * 该方法从数据库中读取短信节点配置，并将它们缓存到Redis中
     * 这样做是为了提高配置数据的读取效率，避免频繁访问数据库
     */
    public void loadingConfigCache() {
        try {
            // 从数据库中选择所有短信节点数据对象
            List<SmsNodeDO> smsNodeDOList = smsNodeMapper.selectList();
            for (SmsNodeDO smsNodeDO : smsNodeDOList) {
                // 构建缓存键
                String key = buildCacheKey(smsNodeDO);
                // 将对象转换为JSON字符串作为缓存值
                String value = JSON.toJSONString(smsNodeDO);
                // 将键值对存入Redis缓存
                redisHelper.set(key, value);
                redisHelper.set(SYSTEM_SMS_NODE_KEY + smsNodeDO.getId(), value);
                // 记录日志信息
//                log.info("Cache key: {}, value: {}", key, value);
            }
        } catch (Exception e) {
            // 记录错误日志信息
            log.error("Error loading config cache", e);
        }
    }

    /**
     * 构建缓存键
     * 根据SmsNodeDO对象的属性生成一个唯一的缓存键
     * 这是为了确保每个配置项在缓存中都有一个唯一的标识
     *
     * @param smsNodeDO SmsNodeDO 对象
     * @return 缓存键
     */
    @Override
    public String buildCacheKey(SmsNodeDO smsNodeDO) {
        // 使用格式化字符串构建缓存键，SYSTEM_SMS_NODE_KEY:短信节点:是否匹配运输方式:运输方式:是否多订单:国家代码:启用状态
        return String.format("%s%s:%s:%s:%s:%s:%s",
                SYSTEM_SMS_NODE_KEY,
                smsNodeDO.getNodeValue(),
                smsNodeDO.getIsTransport(),
                smsNodeDO.getTransportId(),
                smsNodeDO.getIsOrders(),
                smsNodeDO.getCountryCode(),
                smsNodeDO.getStatus());
    }

    /**
     * 每半小时执行一次
     * 短信重发定时任务
     */
    @Scheduled(cron = "0 0/30 * * * ?")
    public void scheduleReceiveRefresh() {
        try {
            // 获取需要重发短信的日志列表
            List<SmsLogDO> smsLogs = smsLogService.selectReceiveList();
            if (smsLogs != null) {
                for (SmsLogDO smsLog : smsLogs) {
                    try {
                        processSmsLog(smsLog);
                    } catch (Exception e) {
                        log.error("Error processing sms log: {}", smsLog, e);
                    }
                }
                // 批量更新短信日志
                smsLogService.updateBatchSmsLog(smsLogs);
            }
        } catch (Exception e) {
            log.error("Error in scheduleReceiveRefresh", e);
        }
    }

    /**
     * 处理短信日志
     *
     * @param smsLog 短信日志对象
     */
    private void processSmsLog(SmsLogDO smsLog) {
        // 根据渠道ID获取短信客户端
        SmsClient smsClient = smsClientFactory.getSmsClient(smsLog.getChannelId());
        // 获取短信接收状态
        SmsLogDTO receiveStatus = smsClient.getReceiveStatus(SmsLogConvert.INSTANCE.toDto(smsLog));
        smsLog.setReceiveStatus(receiveStatus.getReceiveStatus());
        // 接收失败，则重发
        if (ReceiveStatusEnum.Receive_STATUS_20.getValue() == receiveStatus.getReceiveStatus()) {
            smsLog.setReceiveTime(receiveStatus.getReceiveTime());
            smsLog.setNum(1);
            // 从Redis中获取短信节点信息
            String node = redisHelper.get(SYSTEM_SMS_NODE_KEY + smsLog.getNodeId());
            if (node == null) {
                return;
            }
            // 解析短信节点信息
            SmsNodeDO smsNodeDO = JSON.parseObject(node, SmsNodeDO.class);
            // 调用短信发送服务重新发送短信
            smsSendService.sendSingleSmsV2(smsLog.getMobile(), smsLog.getUserId(), smsLog.getUserType(), smsNodeDO.getNodeValue(),
                    smsNodeDO.getCountryCode(), smsNodeDO.getIsOrders(), smsNodeDO.getIsTransport(), smsNodeDO.getTransportId(),
                    smsLog.getMessageType(), smsLog.getTemplateParams(), smsLog.getId(), smsLog.getNodeTemplateSn() + 1);
        }
    }

    @Override
    public Long createSmsNode(SmsNodeCreateReqVO createReqVO) {
        // 插入
        SmsNodeDO smsNode = SmsNodeConvert.INSTANCE.convert(createReqVO);
        smsNodeMapper.insert(smsNode);
        // 设置新的缓存
        redisHelper.set(buildCacheKey(smsNode), JSON.toJSONString(smsNode));
        redisHelper.set(SYSTEM_SMS_NODE_KEY + smsNode.getId(), JSON.toJSONString(smsNode));

        // 返回
        return smsNode.getId();
    }

    @Override
    public void updateSmsNode(SmsNodeUpdateReqVO updateReqVO) {
        // 校验存在
        SmsNodeDO smsNodeDO = this.validateSmsNodeExists(updateReqVO.getId());
        // 更新
        SmsNodeDO updateObj = SmsNodeConvert.INSTANCE.convert(updateReqVO);
        smsNodeMapper.updateById(updateObj);
        // 删除旧的缓存
        redisHelper.delete(buildCacheKey(smsNodeDO));
        // 设置新的缓存
        redisHelper.set(buildCacheKey(updateObj), JSON.toJSONString(updateObj));
        redisHelper.set(SYSTEM_SMS_NODE_KEY + updateObj.getId(), JSON.toJSONString(updateObj));
    }

    @Override
    public void deleteSmsNode(Long id) {
        // 校验存在
        SmsNodeDO smsNodeDO = this.validateSmsNodeExists(id);
        // 删除
        smsNodeMapper.deleteById(id);
        // 删除缓存中的短信节点信息
        redisHelper.delete(buildCacheKey(smsNodeDO));
        redisHelper.delete(SYSTEM_SMS_NODE_KEY + id);
    }

    private SmsNodeDO validateSmsNodeExists(Long id) {
        SmsNodeDO smsNodeDO = smsNodeMapper.selectById(id);
        if (smsNodeDO == null) {
            throw exception(SMS_NODE_NOT_EXISTS);
        }
        return smsNodeDO;
    }

    @Override
    public SmsNodeDO getSmsNode(Long id) {
        return smsNodeMapper.selectById(id);
    }

    @Override
    public List<SmsNodeDO> getSmsNodeList(Collection<Long> ids) {
        return smsNodeMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<SmsNodeDO> getSmsNodePage(SmsNodeQueryVO query, PageVO page) {
        return smsNodeMapper.selectPage(page, query);
    }

    @Override
    public List<SmsNodeDO> getSmsNodeList(SmsNodeQueryVO query) {
        return smsNodeMapper.selectList(query);
    }

}
